#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <iostream>

void calcCubicSplines();
void getSplinePoint(int i, double t/*, double *x, double *y*/);

// INPUT:
const int npoints=4; // 4 points
// matrix C [point] [dimension: 0=x, 1=y]
double C[npoints][2]={{0,0}, {10,10}, {20,9}, {15,0}};
// OUTPUT:
// solution matrix S:
// S [spline number: 0..(npoints-2)] [2 dimensions: x,y] [4 constants: a,b,c,d]
double S[npoints-1][2][4];

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::on_pushButton_clicked()
{
    calcCubicSplines();
    getSplinePoint(2,0.5);
}

// calculate all cubic spline values:
void calcCubicSplines(){
    double w, b[npoints], d[npoints], x[npoints];
    int i, dim;
    int n = npoints-1; // number of splines

    for(dim=0;dim<2;dim++){
        // define d[]:
        d[0]=3.0*(C[1][dim]-C[0][dim]);
        for(i=1;i<n;i++)
            d[i]=3.0*(C[i+1][dim]-C[i-1][dim]);
        d[n]=3.0*(C[n][dim]-C[n-1][dim]);
        // forward sweep: (simplified tridiagonal solution: all a[i]=1 and all c[i]=1)
        b[0]=2.0;
        for(i=1;i<npoints;i++){
            w = 1.0/b[i-1];
            b[i] = 4.0 - w;
            d[i] -= (w*d[i-1]);
        }
        // calculate solution vector x[i] = D[i]:
        // (Wikipedia x[] = Wolfram D[])
        x[n]=d[n]/b[n];
        for(i=n-1;i>=0;i--)
            x[i]=(d[i]-x[i+1])/b[i];
        // calculate spline S[i][dim] a, b, c and d:
        for(i=0;i<n;i++){
            S[i][dim][0]=C[i][dim];
            S[i][dim][1]=x[i];
            S[i][dim][2]=3.0*(C[i+1][dim]-C[i][dim]) - 2.0*x[i] - x[i+1];
            S[i][dim][3]=2.0*(C[i][dim]-C[i+1][dim]) + x[i] + x[i+1];
        }
    }

}

// get x and y coordinates for spline i (i = 0..npoints-2) where t in interval [0, 1]:
void getSplinePoint(int i, double t/*, double *x, double *y*/){
    /*(*x)*/ double x = ((S[i][0][3]*t+S[i][0][2])*t+S[i][0][1])*t+S[i][0][0];
    /*(*y)*/ double y = ((S[i][1][3]*t+S[i][1][2])*t+S[i][1][1])*t+S[i][1][0];
    std::cout<< "x:" << x << " y:" << y << std::endl;
}
